/* player_interface.h */
/* Definitions used to interface to the Mitosis game engine */

/* Player_Name:                                             */
/*  This is a global variable that should be defined and    */
/*  initialized by the player's code.                       */
/*  Only the first 16 characters will actually be used.     */
extern char Player_Name [];


/* Square_Contents: */
/*  This enumerated type defines the possible contents of   */
/*  a square on the playing field.                          */
typedef enum Square_Contents
    {
    Empty,
    Wall,
    Friend,
    Foe,
    Food,
    Toxin,
    Toxin_And_Foe,
    Toxin_And_Friend
    } Square_Contents;


/* forware declaration -- see below */
struct Look_Info;

/* Entity_State:                                        */
/*  This data structure is used to pass the state of    */
/*  the current "entity" or life form to the player's   */
/*  next turn routine.  All fields in this structure    */
/*  should be considered read-only.                     */
typedef struct Entity_State
    {
    /* The current round number. */
    int         round_number;

    /* A unique identifier for this entity. */
    int         entity_ID;

    /* The entity's position on the board. */
    int         entity_position_x;
    int         entity_position_y;

    /* The current energy level of this entity. */
    int         energy;

    /* The current sense values for this entity,    */
    /* representing its ability to detect the       */
    /* general contents of its surroundings in a    */
    /* non-directional manner.                      */
    double      sense_of_food;
    double      sense_of_toxin;
    double      sense_of_foes;

    /* A flag that will be true (non-zero) as long as   */
    /* the entity's energy level is > 0; if the flag    */
    /* is zero, the entity is dead and the player       */
    /* routine is being called one last time for it.    */
    int         not_dead_yet_flag;

    /* If a "Look" move was made the previous round,    */
    /* this will point to a Look_Info structure holding */
    /* the contents of the squares surrounding the      */
    /* entity; this structure is read-only and valid    */
    /* only for the duration of the current invocation  */
    /* of the player routine.  (If a "Look" was not     */
    /* made the previous round, this field is NULL.)    */
    struct Look_Info *  look_info_ptr;  

    /* The contents of the entity's current square at   */
    /* the end of the previous round; will only be      */
    /* Food, Toxin, or Empty.                           */
    Square_Contents square_contents;

    } Entity_State;


/* Move: */
/*  This enumerated type defines the possible actions   */
/*  that an entity can take each round.                 */
typedef enum Move
    {
    Stay,
    Move_Up,
    Move_Down,
    Move_Left,
    Move_Right,
    Split,
    Look
    } Move;


/* Size of the message character arrays -- see below */
#define MESSAGE_MAX_LINE_LEN    60

/* Next_Play: */
/*  This data structure is used to indicate what move   */
/*  an entity should perform for this round; the player */
/*  routine will fill in the fields as appropriate.     */
typedef struct Next_Play
    {
    /* The action this entity will take this round.     */
    /* Must be filled in every round.                   */
    Move        play_choice;

    /* If the "play_choice" field is Look, this field   */
    /* contains the number of squares to look in each   */
    /* direction (between 1 and 5).                     */
    int         look_radius;

    /* If the player wants to display a message this    */
    /* round, this field is set to a non-zero value.    */
    int         message_flag;

    /* When "message_flag" is non-zero, these fields    */
    /* must contain null-terminated strings which       */
    /* represent the two lines of message text to be    */
    /* displayed.                                       */
    char        message1 [MESSAGE_MAX_LINE_LEN];
    char        message2 [MESSAGE_MAX_LINE_LEN];
    } Next_Play;


/* Maximum size of the Look_Info "squares" array -- see below */
#define MAX_ARRAY_LENGTH 11

/* Look_Info: */
/*  This data structure contains information about a    */
/*  square region of the game board; the terrain and    */
/*  other life forms in the region are shown.           */
typedef struct Look_Info
    {
    /* This field indicates the number of array elements    */
    /* in each dimension that are valid.  For example,      */
    /* given a look radius of 3, this field will contain    */
    /* the value 7 (3 on either side of the entity, plus    */
    /* the entity's row/column).  This indicates that only  */
    /* the values 0-6 are valid array indices in this case. */
    int             side_length;

    /* These fields indicate the position on the board that */
    /* corresponds to the upper left hand corner of the     */
    /* "squares" array (i.e. squares[0][0]).  Note that     */
    /* negative values can appear in these fields.  For     */
    /* example, if the entity performing the Look operation */
    /* is in the upper left hand corner of the board and a  */
    /* look distance of 1 is used, the values for "board_x" */
    /* and "board_y" will be -1.  The "squares" array will  */
    /* indicate contents of "Wall" for these squares that   */
    /* are off the board.                                   */
    int             board_x;
    int             board_y;

    /* The contents of the board squares surrounding the    */
    /* entity performing the Look operation, represented as */
    /* a 2 dimensional array with indices 0..side_length-1. */
    Square_Contents squares [MAX_ARRAY_LENGTH] [MAX_ARRAY_LENGTH];
    } Look_Info;


/* end of player_interface.h */
